// C Source File
// Created 10/18/2003; 10:10:56 PM
// Special thanks to Thomas Nussbaumer (extgraph library) and David Coz (rotational algorithm)

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define ENABLE_ERROR_RETURN   // Enable Returning Errors to AMS

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define MIN_AMS 205           // Compile for AMS 2.05 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         	// Include All Header Files
#include "extgraph.h"						// Thomas Nussbaumer's and Lionel Debroux's extgraph library
#include "c_sprites.h"					// Enhanced sprite functions
#include "menus.h"							// Special menu functions
#include "astromin_dat.h"				// Custom Astro Miner variables, sprites, and functions

INT_HANDLER save_1;

USER player;
ENEMY enemy_stack[25];
JEWEL curr_jewel;
SPACE_VORTEX vortex;
EXPLOSION explosion_stack[25];

char str1[30], str2[30], str3[30], str4[30], str5[30], str6[30], str7[30], str8[30], str9[30], str0[30], stra[30], strb[30];

void update_ship(void);
void get_input(void);
void push_enemy(int, int, int, int, int);
void push_explosion(int, int, int, int, int);
void update_enemy(void);
void update_jewel(void);
void update_explosion(void);
void update_vortex(void);
void decrease_shield(int);
void die(void);
void update_shield(void);
void update_stats(void);
//void manage_high_scores(void);
//void get_name(char (*)[][], char [], char);
//void get_score(short (*)[], char [], char);
//void move_name(char (*)[][], int);

// Main Function
void _main(void)
{
	randomize();
	
	GrayAdjust(0);
	
	player.x = LCD_HEIGHT/2; player.y = LCD_HEIGHT/2; player.xvec = 0; player.yvec = 0;
	player.angle = 0; player.ship_xy = ship_coordinates; player.line_order = ship_line_order;
	player.score = 0; player.lives = 3; player.active = 1; player.shield = 100;
	
	curr_jewel.x = random(LCD_WIDTH)-8;
	curr_jewel.y = random(2) == 1 ? -16 : LCD_HEIGHT + 16;
	curr_jewel.type = random(3);
	int x = random(LCD_WIDTH-8), y = random(LCD_HEIGHT-8);
	push_explosion(x, y, END, 0, -1);
	curr_jewel.xvec = random(2) == 1 ? 1 : -1;
	curr_jewel.yvec = random(2) == 1 ? 1 : -1;
	curr_jewel.move_speed = random(5);
	curr_jewel.move_speed_save = curr_jewel.move_speed;
	
	vortex.exist = 0;
	vortex.prob = 50;
	vortex.grav_default = 32;
	
	strncpy(str1, "               ", 15);
	strncpy(str2, "                    ", 20);
	strncpy(str3, "                    ", 20);
	strncpy(str4, "    ", 4);
	strncpy(str5, "                    ", 20);
	strncpy(str6, "                    ", 20);
	strncpy(str7, "                    ", 20);
	strncpy(str8, "                    ", 20);
	strncpy(str9, "         ", 9);
	strncpy(str0, "                    ", 20);
	strncpy(stra, "                ", 16);
	strncpy(strb, "     ", 5);
	
	strncpy(str1, "Astro Miner 0.1", 15);
	strncpy(str2, "Start Game", 10);
	strncpy(str3, "Options", 7);
	strncpy(str4, "Help", 4);
	strncpy(str5, "Exit", 4);
	strncpy(str6, "Vortex Probability", 18);
	strncpy(str7, "Number of lives", 15);
	strncpy(str8, " ", 1);
	strncpy(str9, "GAME OVER", 9);
	strncpy(str0, "Score: ", 7);
	
	strncpy(stra, "Gravity Strength", 16);
	strncpy(strb, "About", 5);

	char *main_data[6] = {str1, str2, str3, str4, strb, str5};
	int choice;
	menu:
	choice = menu(150, 90, 5, main_data, m_select);

	if(choice == 1) {
		strncpy(str1, "  ERROR!  ", 6);
		strncpy(str2, "INSUFFICIENT MEMORY", 19);
		char *data[2] = {str1, str2};
		save_1 = GetIntVec(AUTO_INT_1);
		SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
		if(!GraySpriteInit(3840)) {
			SetIntVec(AUTO_INT_1, save_1);
			menu(150, 50, 2, data, m_message);
			ngetchx();
			exit(0);
		}
		if(!(light_buff = malloc(3840))) {
			GraySpriteDeInit();
			SetIntVec(AUTO_INT_1, save_1);
			menu(150, 50, 2, data, m_message);
			ngetchx();
			exit(0);
		}
		if(!(dark_buff = malloc(3840))) {
			free(light_buff);
			GraySpriteDeInit();
			SetIntVec(AUTO_INT_1, save_1);
			menu(150, 50, 2, data, m_message);
			ngetchx();
			exit(0);
		}
		
		clrscr();
		PortSet(light_buff, 239, 127); clrscr();
		PortSet(dark_buff, 239, 127); clrscr();
		PortSet(light_vscreen, 239, 127); clrscr();
		PortSet(dark_vscreen, 239, 127); clrscr();
		
		int i, j;
		for(i=0; i<LCD_WIDTH; i+=16) {
			for(j=0; j<LCD_HEIGHT; j+=16) {
				CSprite16(i, j, 16, jewels_star_background, light_buff, SPRT_XOR);
			}
		}
		
		for(i=0; i<25; i++) {
			enemy_stack[i].type = none;
			explosion_stack[i].frame = 0;
			explosion_stack[i].exist = 0;
		}
		//game cycle
		while(1) {
			FastCopyScreen(light_buff, light_vscreen);
			FastCopyScreen(dark_buff, dark_vscreen);
			get_input();
			update_ship();
			update_enemy();
			update_jewel();
			update_explosion();
			update_vortex();
			update_stats();
			FastCopyScreen(light_vscreen, GetPlane(LIGHT_PLANE));
			FastCopyScreen(dark_vscreen, GetPlane(DARK_PLANE));
		}
	}
	if(choice == 2) {
		char stat_1[3] = {50, 1, 99};
		char stat_2[3] = {player.lives, 1, 5};
		char stat_3[3] = {vortex.grav_default, 1, 50};
		char *option_data[7] = {str3, str6, str7, stra, stat_1, stat_2, stat_3};
		menu(150, 90, 3, option_data, m_stat_change);
		vortex.prob = 100-option_data[4][m_NUM];
		player.lives = option_data[5][m_NUM] - 1;
		vortex.grav_default = option_data[6][m_NUM];
		goto menu;
	}
	if(choice == 3) {
		char *help_dat[2] = {str4, str8};
		menu(150, 90, 2, help_dat, m_message);
		FontSetSys(F_4x6);
		printf_xy(10, 30-5, ">> Press LEFT or RIGHT to turn the ship");
		printf_xy(10, 40-5, ">> Press UP to apply thrust");
		printf_xy(10, 50-5, ">> Try to collect the jewels");
		printf_xy(10, 60-5, ">> Avoid the enemies that follow you");
		printf_xy(10, 70-5, ">> Avoid space vortexes too");
		printf_xy(10, 80-5, ">> Watch your shield bar!");
		ngetchx();
		goto menu;
	}
	if(choice == 4) {
		char *about_dat[2] = {strb, str8};
		menu(150, 90, 2, about_dat, m_message);
		FontSetSys(F_4x6);
		printf_xy(10, 20, "Astro Miner 0.1");
		printf_xy(10, 30, "-Programmer:  Jude Nelson");
		printf_xy(10, 40, "-Graphics:  Jude Nelson");
		printf_xy(10, 50, "-menus.h, c_sprites.h, and astromin_dat.h");
		printf_xy(10, 60, "header files also by Jude Nelson");
		printf_xy(10, 70, "-Special thanks to Thomas Nussbaumer,");
		printf_xy(10, 80, "Lionel Debroux, and David Coz");
		ngetchx();
		goto menu;
	}
}

void get_input(void) {
	static int key_down = 0;
	if(player.active == 1) {
		if(_keytest (RR_LEFT)) {
			clrscr(); 
			player.angle-=6;
			key_down = 1;
		}
		if(_keytest (RR_RIGHT)) {
			clrscr();
			player.angle+=6;
			key_down = 1;
		}
		if(_keytest (RR_CLEAR)) {
			player.lives = -1;
			die();
		}
		if(_keytest (RR_UP)) {
			int magshiftx = -lookup_sin[player.angle]>>3, magshifty = lookup_cos[player.angle]>>3;
			if(player.xvec < -256) player.xvec += 24;
			if(player.xvec > 256) player.xvec -= 24;
			if(player.yvec < -256) player.yvec += 24;
			if(player.yvec > 256) player.yvec -= 24;
			player.xvec += magshiftx;
			player.yvec += magshifty;
		}
		if(!_keytest (RR_UP)) {
			if(player.xvec < 0) {player.xvec += 8; if(player.xvec > 0) player.xvec = 0;}
			if(player.xvec > 0) {player.xvec -= 8; if(player.xvec < 0) player.xvec = 0;}
			if(player.yvec < 0) {player.yvec += 8; if(player.yvec > 0) player.yvec = 0;}
			if(player.yvec > 0) {player.yvec -= 8; if(player.yvec < 0) player.yvec = 0;}
		}
	}
}

void update_ship(void) {
	//This algorithm was derived from David Coz's 3D programming tutorial and modified so that it supported only 2D, yet very fast movement
	if(player.active > 1) player.active--;
	if(player.active == 1) {
		player.x += player.xvec>>6;
		player.y += player.yvec>>6;
		if(player.x < 8) {player.xvec = 0; player.x = 10;}
		if(player.x > LCD_WIDTH-8) {player.xvec = 0; player.x = LCD_WIDTH - 10;}
		if(player.y < 8) {player.yvec=0; player.y = 10;}
		if(player.y > LCD_HEIGHT-8) {player.yvec = 0; player.y = LCD_HEIGHT-10;}
		int sin = lookup_sin[player.angle], cos = lookup_cos[player.angle];
		int maxnum=29,maxline=34;
		int scr[34][2];
		int i;
		for(i=0; i<maxnum; i++) {
	
			int work_x = player.ship_xy[i].x, work_y = player.ship_xy[i].y;
			int intermediate = work_x;
	  	work_x = (work_x*cos-work_y*sin)>>7;
			work_y = (work_y*cos+intermediate*sin)>>7;
	
	
			scr[i][0] = (1*work_x)/2 + player.x;
			scr[i][1] = (1*work_y)/2 + player.y;
		}
		for(i=0; i<maxline; i++) {
			int ptx1 = scr[player.line_order[i].line1][0];
			int pty1 = scr[player.line_order[i].line1][1];
			int ptx2 = scr[player.line_order[i].line2][0];
			int pty2 = scr[player.line_order[i].line2][1];
			GrayClipLine(ptx1, pty1, ptx2, pty2, light_vscreen, dark_vscreen, A_OR);
		}
	}
}

void push_enemy(int type, int x, int y, int xvec, int yvec) {
	int i;
	for(i=0; i<75; i++) {
		if(enemy_stack[i].type == none) {
			enemy_stack[i].type = type;
			enemy_stack[i].x = x;
			enemy_stack[i].y = y;
			enemy_stack[i].xvec = xvec;
			enemy_stack[i].yvec = yvec;
			break;
		}
	}
}

void update_enemy(void) {
	int i;
	static int timer = 5;
	timer--;
	for(i=0; i<25; i++) {
		if(enemy_stack[i].type != none) {
			if(enemy_stack[i].type == bomb && !vortex.exist) {
				enemy_stack[i].xvec = (player.x - enemy_stack[i].x)>>3;
				enemy_stack[i].yvec = (player.y - enemy_stack[i].y)>>3;
				if(abs(enemy_stack[i].xvec) > 4) enemy_stack[i].xvec = enemy_stack[i].xvec>>1;
				if(abs(enemy_stack[i].yvec) > 4) enemy_stack[i].yvec = enemy_stack[i].yvec>>1;
			}
			if(enemy_stack[i].x < 0) {enemy_stack[i].xvec = 0; enemy_stack[i].x = 2;}
			if(enemy_stack[i].x > LCD_WIDTH) {enemy_stack[i].xvec = 0; enemy_stack[i].x = LCD_WIDTH-10;}
			if(enemy_stack[i].y < 0) {enemy_stack[i].yvec = 0; enemy_stack[i].y = 2;}
			if(enemy_stack[i].y > LCD_HEIGHT) {enemy_stack[i].yvec = 0; enemy_stack[i].y = LCD_HEIGHT-10;}
			
			if(enemy_stack[i].x-5 < player.x+5 && enemy_stack[i].x+8+5 > player.x-5 && enemy_stack[i].y-5 < player.y+5 && enemy_stack[i].y+8+5 > player.y-5) {
				enemy_stack[i].x += enemy_stack[i].xvec;
				enemy_stack[i].y += enemy_stack[i].yvec;
			}
			int j;
			for(j=0; j<25; j++) {
				if(j != i && enemy_stack[j].type != none && enemy_stack[i].x+8 > enemy_stack[j].x-4 && enemy_stack[i].x < enemy_stack[j].x+8+4 && enemy_stack[i].y+8 > enemy_stack[j].y-4 && enemy_stack[i].y < enemy_stack[j].y+8+4) {
					if(enemy_stack[i].x+4 > enemy_stack[j].x) enemy_stack[i].xvec++;
					else enemy_stack[i].xvec--;
					if(enemy_stack[i].y+4 > enemy_stack[j].y) enemy_stack[i].yvec++;
					else enemy_stack[i].yvec--;
				}
				if(j != i && enemy_stack[j].type != none && enemy_stack[i].x+8 > enemy_stack[j].x && enemy_stack[i].x < enemy_stack[j].x+8 && enemy_stack[i].y+8 > enemy_stack[j].y && enemy_stack[i].y < enemy_stack[j].y+8) {
					enemy_stack[i].type = none; enemy_stack[j].type = none;
					push_explosion(enemy_stack[i].x, enemy_stack[i].y, 0, END, 1);
					push_explosion(enemy_stack[j].x, enemy_stack[j].y, 0, END, 1);
				}
			}
			if(vortex.exist) {
				if(enemy_stack[i].x > vortex.x+8) enemy_stack[i].xvec -= vortex.grav_strength>>4;
				else enemy_stack[i].xvec += vortex.grav_strength>>4;
				if(enemy_stack[i].y > vortex.y+8) enemy_stack[i].yvec -= vortex.grav_strength>>4;
				else enemy_stack[i].yvec += vortex.grav_strength>>4;
				
				enemy_stack[i].x += enemy_stack[i].xvec;
				enemy_stack[i].y += enemy_stack[i].yvec;
				
				if(enemy_stack[i].x+8 > vortex.x && enemy_stack[i].x < vortex.x + 16 && enemy_stack[i].y < vortex.y+16 && enemy_stack[i].y+8 > vortex.y) {
					enemy_stack[i].type = none;
					push_explosion(enemy_stack[i].x, enemy_stack[i].y, 0, END, 1);
				}
			}
			if(timer == 0 && player.active == 1) {
				enemy_stack[i].x += enemy_stack[i].xvec>>1;
				enemy_stack[i].y += enemy_stack[i].yvec>>1;
			}
			m_Sprite8(enemy_stack[i].x, enemy_stack[i].y, 8, en_images[enemy_stack[i].type], light_vscreen, dark_vscreen, SPRT_OR);
			
			if(enemy_stack[i].x > player.x-10 && enemy_stack[i].x+8 < player.x+10 && enemy_stack[i].y > player.y-10 && enemy_stack[i].y+8 < player.y+10 && player.active == 1) {
				decrease_shield(5*enemy_stack[i].type);
				push_explosion(enemy_stack[i].x, enemy_stack[i].y, 0, END, 1);
				enemy_stack[i].type = none;
			}
		}
	}
	if(timer == 0) timer = 5;
}

void update_jewel(void) {
	static int timer = 0, x, y;
	if(timer == 0 && curr_jewel.x < player.x+5 && curr_jewel.x+8 > player.x-5 && curr_jewel.y < player.y +5 && curr_jewel.y+8 > player.y - 5) {
		curr_jewel.x = random(LCD_WIDTH)-8;
		curr_jewel.y = random(2) == 1 ? -16 : LCD_HEIGHT + 16;
		curr_jewel.type = random(3);
		x = random(LCD_WIDTH-8), y = random(LCD_HEIGHT-8);
		push_explosion(x, y, END, 0, -1);
		timer=-1;
		curr_jewel.xvec = random(2) == 1 ? 1 : -1;
		curr_jewel.yvec = random(2) == 1 ? 1 : -1;
		curr_jewel.move_speed = random(5)+1;
		curr_jewel.move_speed_save = curr_jewel.move_speed;
		player.score += 50 * (curr_jewel.type + 1);
	}
	if(timer == 8) push_explosion(curr_jewel.x, curr_jewel.y, END, 0, -1);
	if(timer == 0) {
		curr_jewel.move_speed--;
		if(curr_jewel.move_speed == 0) {
			curr_jewel.x += curr_jewel.xvec;
			curr_jewel.y += curr_jewel.yvec;
			curr_jewel.move_speed = curr_jewel.move_speed_save;
			if(curr_jewel.x <= 0) curr_jewel.xvec = abs(curr_jewel.xvec);
			if(curr_jewel.x >= LCD_WIDTH-8) curr_jewel.xvec = -abs(curr_jewel.xvec);
			if(curr_jewel.y <= 0) curr_jewel.yvec = abs(curr_jewel.yvec);
			if(curr_jewel.y >= LCD_HEIGHT-8) curr_jewel.yvec = -abs(curr_jewel.yvec);
		}
		m_Sprite8(curr_jewel.x, curr_jewel.y, 8, images[curr_jewel.type],light_vscreen,dark_vscreen,SPRT_XOR);
	}
	else timer--;
	if(timer < -10) {
		push_enemy(bomb, x, y, 0, 0);
		timer = 9;
	}
}

void push_explosion(int x, int y, int frame, int maxframe, int step) {
	int i;
	for(i=0; i<25; i++) {
		if(explosion_stack[i].exist == none || explosion_stack[i].frame == END || explosion_stack[i].frame == none) {
			explosion_stack[i].exist = 1;
			explosion_stack[i].x = x; 
			explosion_stack[i].y = y;
			explosion_stack[i].frame = frame;
			explosion_stack[i].maxframe = maxframe;
			explosion_stack[i].step = step;
			break;
		}
	}
}

void update_explosion(void) {
	int i;
	for(i=0; i<25; i++) {
		if(explosion_stack[i].exist != 0) {
			if(explosion_stack[i].frame > END || explosion_stack[i].frame < 0) explosion_stack[i].exist = none;
			else {
				m_Sprite8(explosion_stack[i].x, explosion_stack[i].y, 8, explosions[explosion_stack[i].frame], light_vscreen, dark_vscreen, SPRT_XOR);
				explosion_stack[i].frame += explosion_stack[i].step;
			}
		}
	}
}

void update_vortex(void) {
	int randint = random(vortex.prob<<3);
	if(randint == 20 && !vortex.exist && player.active) {
		vortex.x = random(LCD_WIDTH-32);
		vortex.y = random(LCD_HEIGHT-32);
		if(!(vortex.x < player.x + 16 && vortex.x + 32 > player.x - 16 && vortex.y + 32 < player.y + 16 && vortex.y > player.y - 16)) vortex.exist = 1;
		vortex.grav_rad = 50;
		vortex.grav_strength = vortex.grav_default;
		vortex.timer = 100;
		push_explosion(vortex.x+4, vortex.y+4, END, 0, -1);
	}
	if(vortex.exist) {
		vortex.timer--;
		if(vortex.timer == 0) {vortex.exist = 0; push_explosion(vortex.x+4, vortex.y+4, END, 0, -1);}
		if(player.x > vortex.x+8) player.xvec -= vortex.grav_strength/3;
		else player.xvec += vortex.grav_strength/3;
		if(player.y > vortex.y+8) player.yvec -= vortex.grav_strength/3;
		else player.yvec += vortex.grav_strength/3;
		if(vortex.timer < 92) m_Sprite16(vortex.x, vortex.y, 16, jewels_s_space_vortex, light_vscreen, dark_vscreen, SPRT_XOR);
		if(vortex.x > player.x - 16 && vortex.x+16 < player.x + 16 && vortex.y > player.y - 16 && vortex.y + 16 < player.y + 16 && player.active == 1) {
			decrease_shield(vortex.grav_default>>1);
		}
	}
}

void decrease_shield(int dec) {
	player.shield -= dec;
	if(player.shield < 0) die();
}

void update_shield(void) {
	int i;
	for(i=0; i<player.shield; i++) {
		m_Sprite8(0,i,1,shield_bar,light_vscreen,dark_vscreen,SPRT_XOR);
	}
}

void die(void) {
	push_explosion(player.x, player.y, 0, END, 1);
	player.active = 50;
	int j;
	player.xvec = 0;
	player.yvec = 0;
	for(j=0; j<25; j++) {
		if(enemy_stack[j].type == bomb) {
			player.score += 25;
			push_explosion(enemy_stack[j].x, enemy_stack[j].y, 0, END, 1);
			enemy_stack[j].type = none;
		}
	}
	if(vortex.exist) {
		vortex.exist = 0;
		push_explosion(vortex.x+4, vortex.y+4, 0, END, 1);
	}
	player.x = LCD_WIDTH/2;
	player.y = LCD_HEIGHT/2;
	player.angle = 0;
	player.lives--;
	player.shield = 100;
	if(player.lives <= -1) {
		unsigned short i, delay = 20000;
		for(i=0; i<9; i++) {
			while(delay != 0) delay--;
			OSContrastDn();
			delay = 20000;
		}
		GraySpriteDeInit();
		free(light_buff);
		free(dark_buff);
		PortSet(LCD_MEM, 239, 127);
		clrscr();
		FontSetSys(F_8x10);
		char *data[2] = {str9, str0};
		menu(150, 50, 2, data, m_message);
		short x = (LCD_WIDTH/2) - (150/2);
		short y = (LCD_HEIGHT/2) - (50/2);
		short x2 = x + (150 - (150 - 15));
		short y2 = y+6;
		short spacing = 12;
		printf_xy(x2+32+4, y2+(1)*spacing+2, " %u", player.score);
		for(i=8; i>0; i--) {
			while(delay != 0) delay --;
			OSContrastUp();
			delay = 20000;
		}
		SetIntVec(AUTO_INT_1, save_1);
		ngetchx();
		//manage_high_scores();
		exit(0);
	}
}

void update_stats(void) {
	update_shield();
}
//crap that doesn't work
/*
void manage_high_scores(void) {
	//check to see if the high score file exists
	SYM_ENTRY *symptr = SymFindPtr(SYMSTR("astrohsc"),0);
	if(symptr == NULL) {
		//file does not exist...so create it
		FILE *new_hsc_file = fopen("astrohsc","w+t");
		//put the default high score data into it
		fprintf(new_hsc_file, "1:Mr. Nobody:0 ");
		fprintf(new_hsc_file, "2:Mrs. Nobody:0 ");
		fprintf(new_hsc_file, "3:Sr. Nobody:0 ");
		fprintf(new_hsc_file, "4:Fr. Nobody:0 ");
		fprintf(new_hsc_file, "5:Jr. Nobody:0 ");
		//make it a "aHSC" variable
		fputc(0, new_hsc_file);
		fputs("aHSC", new_hsc_file);
		fputc(0, new_hsc_file);
		fputc(OTH_TAG, new_hsc_file);
		//close the file
		fclose(new_hsc_file);
	}
	//unarchive file if it is archived
	EM_moveSymFromExtMem(SYMSTR("astrohsc"),HS_NULL);
	//(re)open the aHSC file to get the names, ranks, and scores
	FILE *hsc = fopen("astrohsc", "rt");
	char buff[30];
	char ranks[5];
	char names[5][30];
	short scores[5];
	int i;
	//copy file records into temporary strings
	for(i=0; i<5; i++) {
		fgets(buff, 30, hsc);
		ranks[i] = buff[0];
		get_name(&names, buff, ranks[i]);
		get_score(&scores, buff, ranks[i]);
	}
	//check and see if the user's score will go into the high score table
	int user_rank=0;
	for(i=0; i<5; i++) {
		if((unsigned short)scores[i] < player.score) {
			user_rank = i;
			break;
		}
	}
	fclose(hsc);
	//was the score high enough?
	if(user_rank) {
		for(i=5; i>user_rank; i--) {
			scores[i] = scores[i-1];
			move_name(&names, i);
		}
		clrscr();
		FontSetSys(F_8x10);
		printf_xy(0,0,"High Score!  Enter your name:");
		for(i=0; i<30; i++) {str1[i] = ' ';}
		FontSetSys(F_6x8);
		inputStr(str1, 0, 12, 30);
		for(i=0; i<30; i++) {names[user_rank][i] = str1[i];}
	}
	FontSetSys(F_4x6);
	int j;
	for(i=0; i<5; i++) {
		for(j=0; j<30; j++) {buff[j] = ' ';}
		for(j=0; j<30; j++) {buff[j] = names[i][j];}
		printf_xy(20,(i+1)*6,"%d  %s  %d",ranks[i], buff, scores[i]);
	}
	ngetchx();
	SymDel(SYMSTR("astrohsc"));
	FILE *new_hsc = fopen("astrohsc", "w+t");
	for(i=0; i<5; i++) {
		for(j=0; j<30; j++) {buff[j] = ' ';}
		for(j=0; j<30; j++) {buff[j] = names[i][j];}
		fprintf(new_hsc, "%d:%s:%d:", ranks[i], buff, scores[i]);
	}
	fclose(new_hsc);
}

void get_name(char (*names)[5][30], char buff[30], char rank) {
	int i=2;
	while(buff[i]!=':') {
		(*names)[(int)rank][i] = buff[i];
		i++;
	}
	for(;i<30;i++) {
		(*names)[(int)rank][i] = ' ';
	}
}

void get_score(short (*scores)[5], char buff[30], char rank) {
	int i=2;
	while(buff[i]!=':') i++;
	i++;
	(*scores)[(int)rank] = atoi((buff+i));
}

void move_name(char (*names)[5][30], int which) {
	int i;
	for(i=0; i<30; i++) {
		(*names)[which][i] = (*names)[which-1][i];
	}
}
*/